module randomSeed
!---------------------------------------------------------------------------------
! Purpose:
!
! Generate SPP needed random  numbers
!
! Author: Yong Wang 06/21/2017
!---------------------------------------------------------------------------------

    use shr_kind_mod, only: r8 => shr_kind_r8, i8 => shr_kind_i8
    use ppgrid,       only: pver, pcols, pverp, begchunk, endchunk
    use abortutils,   only: endrun
    use cam_logfile,  only : iulog


    implicit None
    public
    save
    
    ! Public methods
    public :: random_number_init ! init random numbers 
    public :: random_number_set  ! set random numbers

    real(r8), allocatable, dimension(:,:) :: ranseeds          ! (pcols,begchunk:endchunk)
    real(r8), allocatable, dimension(:,:) :: ranseeds_tot      ! (plon,plat)

contains

    subroutine random_number_init()

        use dyn_grid,     only : get_dyn_grid_parm

        implicit none

        integer :: plon
        integer :: plat
        integer :: istat

        plon = get_dyn_grid_parm('plon')
        plat = get_dyn_grid_parm('plat')


        !Initialize random numbers
        allocate(ranseeds_tot(plon, plat), stat=istat)
        if( istat /= 0 ) then
            write(iulog,*) 'random_number_init: failed to allocate rannumbers_tot; error = ',istat
            call endrun
        end if

        allocate(ranseeds(pcols, begchunk:endchunk), stat=istat)
        if( istat /= 0 ) then
            write(iulog,*) 'random_number_init: failed to allocate rannumbers; error = ',istat
            call endrun
        end if

    end subroutine random_number_init




    subroutine random_number_set()

! ************************************************************************ 
!   Generating spatial correlated random numbers 
! ************************************************************************ 

        use spmd_utils,     only: masterproc
        use phys_grid,      only: scatter_field_to_chunk
        use dyn_grid,       only: get_dyn_grid_parm

        implicit none
       
        integer :: i, j

        integer :: plon
        integer :: plat

        plon = get_dyn_grid_parm('plon')
        plat = get_dyn_grid_parm('plat')

        if (masterproc) then
            call init_random_seed()
            do i = 1, plon
                do j = 1, plat
                    call random_number(ranseeds_tot(i,j))
                end do
            end do
        end if

        call scatter_field_to_chunk(1, 1, 1, plon, ranseeds_tot, ranseeds)
    
    end subroutine random_number_set 


    subroutine init_random_seed()
        use time_manager, only : get_nstep
        integer :: i,n,nstep 
        integer(i8),dimension(:),allocatable :: seed
        call random_seed(size=n)
        allocate(seed(n))
        nstep = get_nstep() 
        nstep = nstep/48*48
        seed=50*(nstep+1)+37*(/(i-1,i=1,n)/)
        call random_seed(PUT=seed)
        deallocate(seed)
    end subroutine init_random_seed

end module randomSeed
