module set_pbuf_lnd
  
use shr_kind_mod,    only : r8 => shr_kind_r8
use ppgrid,          only : pcols, pver, pverp
use physconst,       only : numpft

implicit none
private
save

public lnd_register
public set_pbuf_pbl
public set_pbuf_dcon
public set_pbuf_scon

integer :: pblh_idx, tke_idx, kvh_idx
integer :: icwmrdp_idx, rprddp_idx, nevapr_dpcu_idx, prec_dp_idx, snow_dp_idx
integer :: icwmrsh_idx, rprdsh_idx, nevapr_shcu_idx, rprdtot_idx, cldtop_idx, cldbot_idx, shfrc_idx, prec_sh_idx, snow_sh_idx

contains

subroutine lnd_register()
   use physics_buffer,  only : pbuf_add_field, dtype_r8, dtype_i4
    
   call pbuf_add_field('pblh',  'global', dtype_r8, (/pcols/),        pblh_idx)
   call pbuf_add_field('tke',   'global', dtype_r8, (/pcols, pverp/), tke_idx) 
   call pbuf_add_field('kvh',   'global', dtype_r8, (/pcols, pverp/), kvh_idx) 
  
   call pbuf_add_field('ICWMRDP',    'physpkg', dtype_r8, (/pcols,pver/), icwmrdp_idx)
   call pbuf_add_field('RPRDDP',     'physpkg', dtype_r8, (/pcols,pver/), rprddp_idx)
   call pbuf_add_field('NEVAPR_DPCU','physpkg', dtype_r8, (/pcols,pver/), nevapr_dpcu_idx)
   call pbuf_add_field('PREC_DP',    'physpkg', dtype_r8, (/pcols/), prec_dp_idx)
   call pbuf_add_field('SNOW_DP',    'physpkg', dtype_r8, (/pcols/), snow_dp_idx)
  
   call pbuf_add_field('ICWMRSH',    'physpkg', dtype_r8, (/pcols,pver/),       icwmrsh_idx )
   call pbuf_add_field('RPRDSH',     'physpkg', dtype_r8, (/pcols,pver/),       rprdsh_idx )
   call pbuf_add_field('NEVAPR_SHCU','physpkg', dtype_r8, (/pcols,pver/),       nevapr_shcu_idx )
   call pbuf_add_field('RPRDTOT',    'physpkg', dtype_r8, (/pcols,pver/),       rprdtot_idx )
   call pbuf_add_field('CLDTOP',     'physpkg', dtype_r8, (/pcols/),            cldtop_idx )
   call pbuf_add_field('CLDBOT',     'physpkg', dtype_r8, (/pcols/),            cldbot_idx )
   call pbuf_add_field('shfrc',      'physpkg', dtype_r8, (/pcols,pver/),       shfrc_idx )
   call pbuf_add_field('PREC_SH',    'physpkg', dtype_r8, (/pcols/), prec_sh_idx)
   call pbuf_add_field('SNOW_SH',    'physpkg', dtype_r8, (/pcols/), snow_sh_idx)
end subroutine lnd_register   

subroutine set_pbuf_pbl(pbuf)
   use physics_buffer,  only : pbuf_get_index, pbuf_get_field, pbuf_set_field, physics_buffer_desc
    
   type(physics_buffer_desc), pointer :: pbuf(:)
   
   integer :: pblh_ens_idx, tke_ens_idx, kvh_ens_idx
   integer :: lnd_i
   real(r8), pointer :: pblh_ens(:,:)
   real(r8), pointer :: kvh_ens(:,:,:)
   real(r8), pointer :: tke_ens(:,:,:)
   real(r8) :: pblh(pcols)
   real(r8) :: kvh(pcols, pverp)
   real(r8) :: tke(pcols, pverp)

   pblh_ens_idx = pbuf_get_index('pblh_ens')
   kvh_ens_idx = pbuf_get_index('kvh_ens')
   tke_ens_idx = pbuf_get_index('tke_ens')
   call pbuf_get_field(pbuf, pblh_ens_idx,  pblh_ens)
   call pbuf_get_field(pbuf, kvh_ens_idx,   kvh_ens)
   call pbuf_get_field(pbuf, tke_ens_idx,   tke_ens)
    
   pblh = 0._r8
   kvh  = 0._r8
   tke  = 0._r8
   do lnd_i=2, numpft
      pblh = pblh + (pblh_ens(:,lnd_i)-pblh_ens(:,1))
      kvh = kvh + (kvh_ens(:,:,lnd_i)-kvh_ens(:,:,1))
      tke = tke + (tke_ens(:,:,lnd_i)-tke_ens(:,:,1))
   end do
   pblh = pblh_ens(:,1) + (pblh/numpft)
   kvh = kvh_ens(:,:,1) + (kvh/numpft)
   tke = tke_ens(:,:,1) + (tke/numpft)
   
   call pbuf_set_field(pbuf, pblh_idx, pblh)
   call pbuf_set_field(pbuf, kvh_idx, kvh)
   call pbuf_set_field(pbuf, tke_idx, tke)
end subroutine set_pbuf_pbl

subroutine set_pbuf_dcon(pbuf)
   use physics_buffer,  only : pbuf_get_index, pbuf_get_field, pbuf_set_field, physics_buffer_desc
   
   type(physics_buffer_desc), pointer :: pbuf(:)
   
   integer :: lnd_i
   real(r8), pointer :: icwmrdp_ens(:,:,:)
   real(r8), pointer :: rprddp_ens(:,:,:)
   real(r8), pointer :: evapcdp_ens(:,:,:)
   real(r8), pointer :: prec_dp_ens(:,:)
   real(r8), pointer :: snow_dp_ens(:,:)
   real(r8) :: icwmrdp(pcols,pver)                ! in cloud water mixing ratio, deep convection
   real(r8) :: rprddp(pcols,pver)                 ! rain production, deep convection
   real(r8) :: evapcdp(pcols,pver)  ! Evaporation rate of deep    convective precipitation >=0.
   real(r8) :: prec_dp(pcols)
   real(r8) :: snow_dp(pcols)

   call pbuf_get_field(pbuf, pbuf_get_index('ICWMRDP_ENS'), icwmrdp_ens )
   call pbuf_get_field(pbuf, pbuf_get_index('RPRDDP_ENS'), rprddp_ens )
   call pbuf_get_field(pbuf, pbuf_get_index('NEVAPR_DPCU_ENS'), evapcdp_ens )
   call pbuf_get_field(pbuf, pbuf_get_index('PREC_DP_ENS'), prec_dp_ens )
   call pbuf_get_field(pbuf, pbuf_get_index('SNOW_DP_ENS'), snow_dp_ens )
    
   icwmrdp = 0._r8
   rprddp = 0._r8
   evapcdp = 0._r8
   prec_dp = 0._r8
   snow_dp = 0._r8
   do lnd_i = 2, numpft
      icwmrdp  = icwmrdp + (icwmrdp_ens(:,:,lnd_i)-icwmrdp_ens(:,:,1))  
      rprddp  = rprddp + (rprddp_ens(:,:,lnd_i)-rprddp_ens(:,:,1))  
      evapcdp  = evapcdp + (evapcdp_ens(:,:,lnd_i)-evapcdp_ens(:,:,1))  
      prec_dp  = prec_dp + (prec_dp_ens(:,lnd_i)-prec_dp_ens(:,1))  
      snow_dp  = snow_dp + (snow_dp_ens(:,lnd_i)-snow_dp_ens(:,1))  
   end do
   icwmrdp = icwmrdp_ens(:,:,1) + (icwmrdp/numpft)
   rprddp = rprddp_ens(:,:,1) + (rprddp/numpft)
   evapcdp = evapcdp_ens(:,:,1) + (evapcdp/numpft)
   prec_dp = prec_dp_ens(:,1) + (prec_dp/numpft)
   snow_dp = snow_dp_ens(:,1) + (snow_dp/numpft)
   
   call pbuf_set_field(pbuf, icwmrdp_idx, icwmrdp)
   call pbuf_set_field(pbuf, rprddp_idx, rprddp)
   call pbuf_set_field(pbuf, nevapr_dpcu_idx, evapcdp)
   call pbuf_set_field(pbuf, prec_dp_idx, prec_dp)
   call pbuf_set_field(pbuf, snow_dp_idx, snow_dp)
end subroutine set_pbuf_dcon

subroutine set_pbuf_scon(lchnk, ncol, pbuf)
   use physics_buffer,  only : pbuf_get_index, pbuf_get_field, pbuf_set_field, physics_buffer_desc
   
   type(physics_buffer_desc), pointer :: pbuf(:)
   integer,  intent(in) :: lchnk, ncol
   
   integer :: lnd_i, i
   real(r8), pointer :: icwmrsh_ens(:,:,:)
   real(r8), pointer :: rprdsh_ens(:,:,:)
   real(r8), pointer :: evapcsh_ens(:,:,:)
   real(r8), pointer :: rprdtot_ens(:,:,:)
   real(r8), pointer :: shfrc_ens(:,:,:)
   real(r8), pointer :: cldtop_ens(:,:)
   real(r8), pointer :: cldbot_ens(:,:)
   real(r8), pointer :: prec_sh_ens(:,:)
   real(r8), pointer :: snow_sh_ens(:,:)
   real(r8) :: icwmrsh(pcols,pver)                ! in cloud water mixing ratio, deep convection
   real(r8) :: rprdsh(pcols,pver)                 ! rain production, deep convection
   real(r8) :: evapcsh(pcols,pver)  ! Evaporation rate of deep    convective precipitation >=0.
   real(r8) :: rprdtot(pcols,pver)                 ! rain production, deep convection
   real(r8) :: shfrc(pcols,pver)
   real(r8) :: cldtop(pcols)                 ! rain production, deep convection
   real(r8) :: cldbot(pcols)                 ! rain production, deep convection
   real(r8) :: prec_sh(pcols)
   real(r8) :: snow_sh(pcols)

   call pbuf_get_field(pbuf, pbuf_get_index('ICWMRSH_ENS'), icwmrsh_ens )
   call pbuf_get_field(pbuf, pbuf_get_index('RPRDSH_ENS'), rprdsh_ens )
   call pbuf_get_field(pbuf, pbuf_get_index('NEVAPR_SHCU_ENS'), evapcsh_ens )
   call pbuf_get_field(pbuf, pbuf_get_index('RPRDTOT_ENS'), rprdtot_ens )
   call pbuf_get_field(pbuf, pbuf_get_index('CLDTOP_ENS'), cldtop_ens )
   call pbuf_get_field(pbuf, pbuf_get_index('CLDBOT_ENS'), cldbot_ens )
   call pbuf_get_field(pbuf, pbuf_get_index('shfrc_ens'),  shfrc_ens )
   call pbuf_get_field(pbuf, pbuf_get_index('PREC_SH_ENS'), prec_sh_ens )
   call pbuf_get_field(pbuf, pbuf_get_index('SNOW_SH_ENS'), snow_sh_ens )
    
   icwmrsh = 0._r8
   rprdsh = 0._r8
   evapcsh = 0._r8
   rprdtot = 0._r8
   shfrc = 0._r8
   prec_sh = 0._r8
   snow_sh = 0._r8
   cldtop = cldtop_ens(:,1)
   cldbot = cldbot_ens(:,1)
   do lnd_i = 2, numpft
      icwmrsh  = icwmrsh + (icwmrsh_ens(:,:,lnd_i)-icwmrsh_ens(:,:,1))  
      rprdsh  = rprdsh + (rprdsh_ens(:,:,lnd_i)-rprdsh_ens(:,:,1))  
      evapcsh  = evapcsh + (evapcsh_ens(:,:,lnd_i)-evapcsh_ens(:,:,1))  
      rprdtot  = rprdtot + (rprdtot_ens(:,:,lnd_i)-rprdtot_ens(:,:,1))
      shfrc   = shfrc + (shfrc_ens(:,:,lnd_i)-shfrc_ens(:,:,1))
      prec_sh  = prec_sh + (prec_sh_ens(:,lnd_i)-prec_sh_ens(:,1))  
      snow_sh  = snow_sh + (snow_sh_ens(:,lnd_i)-snow_sh_ens(:,1))  
      do i = 1, ncol
         if(cldtop_ens(i,lnd_i)<cldtop(i)) cldtop(i)=cldtop_ens(i,lnd_i) 
         if(cldbot_ens(i,lnd_i)>cldbot(i)) cldbot(i)=cldbot_ens(i,lnd_i) 
      end do
   end do
   icwmrsh = icwmrsh_ens(:,:,1) + (icwmrsh/numpft)
   rprdsh = rprdsh_ens(:,:,1) + (rprdsh/numpft)
   evapcsh = evapcsh_ens(:,:,1) + (evapcsh/numpft)
   rprdtot = rprdtot_ens(:,:,1) + (rprdtot/numpft)
   shfrc = shfrc_ens(:,:,1) + (shfrc/numpft)
   prec_sh = prec_sh_ens(:,1) + (prec_sh/numpft)
   snow_sh = snow_sh_ens(:,1) + (snow_sh/numpft)
   
   call pbuf_set_field(pbuf, icwmrsh_idx, icwmrsh)
   call pbuf_set_field(pbuf, rprdsh_idx, rprdsh)
   call pbuf_set_field(pbuf, nevapr_shcu_idx, evapcsh)
   call pbuf_set_field(pbuf, rprdtot_idx, rprdtot)
   call pbuf_set_field(pbuf, cldtop_idx, cldtop)
   call pbuf_set_field(pbuf, cldbot_idx, cldbot)
   call pbuf_set_field(pbuf, shfrc_idx, shfrc)
   call pbuf_set_field(pbuf, prec_sh_idx, prec_sh)
   call pbuf_set_field(pbuf, snow_sh_idx, snow_sh)
end subroutine set_pbuf_scon

end module set_pbuf_lnd

