!----------------------------------------------------------------------------------
! The module is added by Wenqi Sun 2019/08
! The module is used for sampling from truncated normal distribution
! The module is modified from https://people.sc.fsu.edu/~jburkardt/f_src/truncated_normal/truncated_normal.html
!----------------------------------------------------------------------------------
module truncated_normal
    implicit none
    private
    public :: truncated_normal_ab_sample
    public :: truncated_normal_ab_test

contains

    function r8_uniform_01 ( seed )
        implicit none
        
        integer ( kind = 4 ), parameter :: i4_huge = 2147483647
        integer ( kind = 4 ) k
        real ( kind = 8 ) r8_uniform_01
        integer ( kind = 4 ) seed
        
        if ( seed == 0 ) then
        write ( *, '(a)' ) ' '
        write ( *, '(a)' ) 'R8_UNIFORM_01 - Fatal error!'
        write ( *, '(a)' ) '  Input value of SEED = 0.'
            stop 1
        end if
        
        k = seed / 127773
        seed = 16807 * ( seed - k * 127773 ) - k * 2836
        
        if ( seed < 0 ) then
        seed = seed + i4_huge
        end if

        r8_uniform_01 = real ( seed, kind = 8 ) * 4.656612875D-10
    end function r8_uniform_01

    function r8poly_value_horner ( m, c, x )
        implicit none
        
        integer ( kind = 4 ) m
        real ( kind = 8 ) c(0:m)
        integer ( kind = 4 ) i
        real ( kind = 8 ) r8poly_value_horner
        real ( kind = 8 ) value
        real ( kind = 8 ) x
                                    
        value = c(m)
        do i = m - 1, 0, -1
            value = value * x + c(i)
        end do
                                                                
        r8poly_value_horner = value
    end function r8poly_value_horner 

    subroutine normal_01_pdf ( x, pdf )
        implicit none
    
        real ( kind = 8 ) pdf
        real ( kind = 8 ), parameter :: r8_pi = 3.141592653589793D+00
        real ( kind = 8 ) x
    
        pdf = exp ( -0.5D+00 * x * x ) / sqrt ( 2.0D+00 * r8_pi )
    end subroutine normal_01_pdf

    subroutine normal_01_cdf ( x, cdf )
        implicit none
            
        real ( kind = 8 ), parameter :: a1 = 0.398942280444D+00
        real ( kind = 8 ), parameter :: a2 = 0.399903438504D+00
        real ( kind = 8 ), parameter :: a3 = 5.75885480458D+00
        real ( kind = 8 ), parameter :: a4 = 29.8213557808D+00
        real ( kind = 8 ), parameter :: a5 = 2.62433121679D+00
        real ( kind = 8 ), parameter :: a6 = 48.6959930692D+00
        real ( kind = 8 ), parameter :: a7 = 5.92885724438D+00
        real ( kind = 8 ), parameter :: b0 = 0.398942280385D+00
        real ( kind = 8 ), parameter :: b1 = 3.8052D-08
        real ( kind = 8 ), parameter :: b2 = 1.00000615302D+00
        real ( kind = 8 ), parameter :: b3 = 3.98064794D-04
        real ( kind = 8 ), parameter :: b4 = 1.98615381364D+00
        real ( kind = 8 ), parameter :: b5 = 0.151679116635D+00
        real ( kind = 8 ), parameter :: b6 = 5.29330324926D+00
        real ( kind = 8 ), parameter :: b7 = 4.8385912808D+00
        real ( kind = 8 ), parameter :: b8 = 15.1508972451D+00
        real ( kind = 8 ), parameter :: b9 = 0.742380924027D+00
        real ( kind = 8 ), parameter :: b10 = 30.789933034D+00
        real ( kind = 8 ), parameter :: b11 = 3.99019417011D+00
        real ( kind = 8 ) cdf
        real ( kind = 8 ) q
        real ( kind = 8 ) x
        real ( kind = 8 ) y
            
        if ( abs ( x ) <= 1.28D+00 ) then
            y = 0.5D+00 * x * x
            q = 0.5D+00 - abs ( x ) * ( a1 - a2 * y / ( y + a3 - a4 / ( y + a5 &
            + a6 / ( y + a7 ) ) ) )
        else if ( abs ( x ) <= 12.7D+00 ) then
            y = 0.5D+00 * x * x
            q = exp ( - y ) * b0 / ( abs ( x ) - b1 &
            + b2 / ( abs ( x ) + b3 &
            + b4 / ( abs ( x ) - b5 &
            + b6 / ( abs ( x ) + b7 &
            - b8 / ( abs ( x ) + b9 &
            + b10 / ( abs ( x ) + b11 ) ) ) ) ) )
        else    
            q = 0.0D+00
        end if
            
        if ( x < 0.0D+00 ) then
            cdf = q
        else
            cdf = 1.0D+00 - q
        end if
    end subroutine normal_01_cdf
        
    subroutine normal_01_cdf_inv ( p, x )
        implicit none
        
        real ( kind = 8 ), parameter, dimension ( 8 ) :: a = (/ &
            3.3871328727963666080D+00, &
            1.3314166789178437745D+02, &
            1.9715909503065514427D+03, &
            1.3731693765509461125D+04, &
            4.5921953931549871457D+04, &
            6.7265770927008700853D+04, &
            3.3430575583588128105D+04, &
            2.5090809287301226727D+03 /)
        real ( kind = 8 ), parameter, dimension ( 8 ) :: b = (/ &
            1.0D+00, &
            4.2313330701600911252D+01, &
            6.8718700749205790830D+02, &
            5.3941960214247511077D+03, &
            2.1213794301586595867D+04, &
            3.9307895800092710610D+04, &
            2.8729085735721942674D+04, &
            5.2264952788528545610D+03 /)
        real ( kind = 8 ), parameter, dimension ( 8 ) :: c = (/ &
            1.42343711074968357734D+00, &
            4.63033784615654529590D+00, &
            5.76949722146069140550D+00, &
            3.64784832476320460504D+00, &
            1.27045825245236838258D+00, &
            2.41780725177450611770D-01, &
            2.27238449892691845833D-02, &
            7.74545014278341407640D-04 /)
        real ( kind = 8 ), parameter :: const1 = 0.180625D+00
        real ( kind = 8 ), parameter :: const2 = 1.6D+00
        real ( kind = 8 ), parameter, dimension ( 8 ) :: d = (/ &
            1.0D+00, &
            2.05319162663775882187D+00, &
            1.67638483018380384940D+00, &
            6.89767334985100004550D-01, &
            1.48103976427480074590D-01, &
            1.51986665636164571966D-02, &
            5.47593808499534494600D-04, &
            1.05075007164441684324D-09 /)
        real ( kind = 8 ), parameter, dimension ( 8 ) :: e = (/ &
            6.65790464350110377720D+00, &
            5.46378491116411436990D+00, &
            1.78482653991729133580D+00, &
            2.96560571828504891230D-01, &
            2.65321895265761230930D-02, &
            1.24266094738807843860D-03, &
            2.71155556874348757815D-05, &
            2.01033439929228813265D-07 /)
        real ( kind = 8 ), parameter, dimension ( 8 ) :: f = (/ &
            1.0D+00, &
            5.99832206555887937690D-01, &
            1.36929880922735805310D-01, &
            1.48753612908506148525D-02, &
            7.86869131145613259100D-04, &
            1.84631831751005468180D-05, &
            1.42151175831644588870D-07, &
            2.04426310338993978564D-15 /)
        real ( kind = 8 ) p
        real ( kind = 8 ) q
        real ( kind = 8 ) r
        real ( kind = 8 ), parameter :: split1 = 0.425D+00
        real ( kind = 8 ), parameter :: split2 = 5.0D+00
        real ( kind = 8 ) x
        
        if ( p <= 0.0D+00 ) then
            x = - huge ( x )
            return
        end if
        
        if ( 1.0D+00 <= p ) then
            x = huge ( x )
            return
        end if
        
        q = p - 0.5D+00
        if ( abs ( q ) <= split1 ) then
            r = const1 - q * q
            x = q * r8poly_value_horner ( 7, a, r ) &
                / r8poly_value_horner ( 7, b, r )
        else
            if ( q < 0.0D+00 ) then
                r = p
            else
                r = 1.0D+00 - p
            end if
        
            if ( r <= 0.0D+00 ) then
                x = huge ( x )
            else
                r = sqrt ( - log ( r ) )
                if ( r <= split2 ) then
                    r = r - const2
                    x = r8poly_value_horner ( 7, c, r ) &
                    / r8poly_value_horner ( 7, d, r )
                else
                    r = r - split2
                    x = r8poly_value_horner ( 7, e, r ) &
                    / r8poly_value_horner ( 7, f, r )
                end if
            end if
        
            if ( q < 0.0D+00 ) then
                x = -x
            end if
        end if
    end subroutine normal_01_cdf_inv

    subroutine truncated_normal_ab_sample ( mu, sigma, a, b, seed, x )
        implicit none
        
        real ( kind = 8 ) a
        real ( kind = 8 ) alpha
        real ( kind = 8 ) alpha_cdf
        real ( kind = 8 ) b
        real ( kind = 8 ) beta
        real ( kind = 8 ) beta_cdf
        real ( kind = 8 ) mu
        real ( kind = 8 ) sigma
        integer ( kind = 4 ) seed
        real ( kind = 8 ) u
        real ( kind = 8 ) x
        real ( kind = 8 ) xi
        real ( kind = 8 ) xi_cdf
        
        alpha = ( a - mu ) / sigma
        beta = ( b - mu ) / sigma
        
        call normal_01_cdf ( alpha, alpha_cdf )
        call normal_01_cdf ( beta, beta_cdf )
        
        u = r8_uniform_01 ( seed )
        xi_cdf = alpha_cdf + u * ( beta_cdf - alpha_cdf )
        call normal_01_cdf_inv ( xi_cdf, xi )
        
        x = mu + sigma * xi
    end subroutine truncated_normal_ab_sample

    subroutine truncated_normal_ab_mean ( mu, sigma, a, b, mean )
        implicit none
    
        real ( kind = 8 ) a
        real ( kind = 8 ) alpha
        real ( kind = 8 ) alpha_cdf
        real ( kind = 8 ) alpha_pdf
        real ( kind = 8 ) b
        real ( kind = 8 ) beta
        real ( kind = 8 ) beta_cdf
        real ( kind = 8 ) beta_pdf
        real ( kind = 8 ) mean
        real ( kind = 8 ) mu
        real ( kind = 8 ) sigma
        
        alpha = ( a - mu ) / sigma
        beta = ( b - mu ) / sigma
        
        call normal_01_cdf ( alpha, alpha_cdf )
        call normal_01_cdf ( beta, beta_cdf )
        
        call normal_01_pdf ( alpha, alpha_pdf )
        call normal_01_pdf ( beta, beta_pdf )
        
        mean = mu + sigma * ( alpha_pdf - beta_pdf ) / ( beta_cdf - alpha_cdf )
    end subroutine truncated_normal_ab_mean

    subroutine truncated_normal_ab_variance ( mu, sigma, a, b, variance )
        implicit none
    
        real ( kind = 8 ) a
        real ( kind = 8 ) alpha
        real ( kind = 8 ) alpha_cdf
        real ( kind = 8 ) alpha_pdf
        real ( kind = 8 ) b
        real ( kind = 8 ) beta
        real ( kind = 8 ) beta_cdf
        real ( kind = 8 ) beta_pdf
        real ( kind = 8 ) mu
        real ( kind = 8 ) sigma
        real ( kind = 8 ) variance
    
        alpha = ( a - mu ) / sigma
        beta = ( b - mu ) / sigma
    
        call normal_01_pdf ( alpha, alpha_pdf )
        call normal_01_pdf ( beta, beta_pdf )
    
        call normal_01_cdf ( alpha, alpha_cdf )
        call normal_01_cdf ( beta, beta_cdf )
    
        variance = sigma * sigma * ( 1.0D+00 &
          + ( alpha * alpha_pdf - beta * beta_pdf ) / ( beta_cdf - alpha_cdf ) &
          - ( ( alpha_pdf - beta_pdf ) / ( beta_cdf - alpha_cdf ) ) ** 2 )
    end subroutine truncated_normal_ab_variance

    subroutine truncated_normal_ab_test(mu, sigma, a, b, seed, sample_num, verbose, mean_error, std_error)
        real(8),intent(in) :: a,b,mu,sigma
        integer(4),intent(in) :: seed, sample_num
        logical,intent(in) :: verbose
        real(8),intent(out) :: mean_error, std_error
      
        integer(4) :: i
        real(8) :: x(sample_num),mean,variance,xmax,xmin,mean_s,variance_s
    
    
        call truncated_normal_ab_mean ( mu, sigma, a, b, mean )
        call truncated_normal_ab_variance ( mu, sigma, a, b, variance )
    
        if(verbose) then
            write ( *, '(a,g14.6)' ) '  PDF mean =        ', mean
            write ( *, '(a,g14.6)' ) '  PDF standard deviation =    ', sqrt(variance)
            write ( *, '(a,g14.6)' ) '  PDF maximum =  ', b
            write ( *, '(a,g14.6)' ) '  PDF minimum =  ', a
        end if
    
        do i=1,sample_num
            call truncated_normal_ab_sample ( mu, sigma, a, b, seed, x(i) )
        end do
    
        mean_s = sum ( x(1:sample_num) ) / real ( sample_num, kind = 8 )
        variance_s = sum ( ( x(1:sample_num) - mean_s ) ** 2 ) / real ( sample_num-1, kind = 8 )
        xmax = maxval(x(1:sample_num))
        xmin = minval(x(1:sample_num))
    
        if(verbose) then
            write ( *, '(a)' ) ''
            write ( *, '(a,g14.6)' ) '  Sample mean =     ', mean_s
            write ( *, '(a,g14.6)' ) '  Sample standard deviation = ', sqrt(variance_s)
            write ( *, '(a,g14.6)' ) '  Sample maximum =  ', xmax
            write ( *, '(a,g14.6)' ) '  Sample minimum =  ', xmin 
        end if
    
        mean_error = abs(mean_s-mean)/mean
        if(.not. isnan(variance)) then
            std_error = abs(sqrt(variance_s)-sqrt(variance))/sqrt(variance)
        else 
            if(variance_s .eq. 0) then
                std_error = 0
            else
                std_error = 1
            end if
        end if
    
        if(verbose) then
            write ( *, '(a)' ) ''
            write ( *, '(a,f5.2,a)' ) '  mean error percentage =     ', mean_error*100, '%'
            write ( *, '(a,f5.2,a)' ) '  standard deviation error percentage= ', std_error*100, '%'
        end if
    end subroutine truncated_normal_ab_test

end module truncated_normal





    
